@@ -1,12 +1,10 @@
-use std::env;
-
 // Don't run UI tests on emulated environment.
 #[cfg(not(target_os = "android"))]
 #[test]
 fn ui() {
   // This environment variable tells build.rs that we're running trybuild tests,
   // so it won't rebuild V8.
-  env::set_var("DENO_TRYBUILD", "1");
+  std::env::set_var("DENO_TRYBUILD", "1");
 
   let t = trybuild::TestCases::new();
   t.compile_fail("tests/compile_fail/*.rs");
