@@ -53,6 +53,18 @@ class CodeDataContainer : public HeapObject {
   // is deterministic.
   inline void clear_padding();
 
+  //
+  // A collection of getters and predicates that are used by respective methods
+  // on Code object. They are defined here mostly because they operate on the
+  // writable state of the respective Code object.
+  //
+
+  inline bool can_have_weak_objects() const;
+  inline void set_can_have_weak_objects(bool value);
+
+  inline bool marked_for_deoptimization() const;
+  inline void set_marked_for_deoptimization(bool flag);
+
   // Back-reference to the Code object.
   // Available only when V8_EXTERNAL_CODE_SPACE is defined.
   DECL_GETTER(code, Code)
@@ -82,9 +94,20 @@ class CodeDataContainer : public HeapObject {
 
   inline void AllocateExternalPointerEntries(Isolate* isolate);
 
+  // Initializes internal flags field which stores cached values of some
+  // properties of the respective Code object.
+  // Available only when V8_EXTERNAL_CODE_SPACE is enabled.
+  inline void initialize_flags(CodeKind kind, Builtin builtin_id);
+
   // Alias for code_entry_point to make it API compatible with Code.
   inline Address InstructionStart() const;
 
+  // Alias for code_entry_point to make it API compatible with Code.
+  inline Address raw_instruction_start();
+
+  // Alias for code_entry_point to make it API compatible with Code.
+  inline Address entry() const;
+
 #ifdef V8_EXTERNAL_CODE_SPACE
   //
   // A collection of getters and predicates that forward queries to associated
@@ -94,8 +117,32 @@ class CodeDataContainer : public HeapObject {
   inline CodeKind kind() const;
   inline Builtin builtin_id() const;
   inline bool is_builtin() const;
+
+  inline bool is_optimized_code() const;
+  inline bool is_wasm_code() const;
+
+  // Testers for interpreter builtins.
   inline bool is_interpreter_trampoline_builtin() const;
 
+  // Testers for baseline builtins.
+  inline bool is_baseline_trampoline_builtin() const;
+  inline bool is_baseline_leave_frame_builtin() const;
+
+  // Tells whether the code checks the optimization marker in the function's
+  // feedback vector.
+  inline bool checks_optimization_marker() const;
+
+  // Tells whether the outgoing parameters of this code are tagged pointers.
+  inline bool has_tagged_outgoing_params() const;
+
+  // [is_turbofanned]: Tells whether the code object was generated by the
+  // TurboFan optimizing compiler.
+  inline bool is_turbofanned() const;
+
+  // [is_off_heap_trampoline]: For kind BUILTIN tells whether
+  // this is a trampoline to an off-heap builtin.
+  inline bool is_off_heap_trampoline() const;
+
   DECL_GETTER(deoptimization_data, FixedArray)
   DECL_GETTER(bytecode_or_interpreter_data, HeapObject)
   DECL_GETTER(source_position_table, ByteArray)
@@ -110,23 +157,25 @@ class CodeDataContainer : public HeapObject {
   DECL_VERIFIER(CodeDataContainer)
 
 // Layout description.
-#define CODE_DATA_FIELDS(V)                                     \
-  /* Strong pointer fields. */                                  \
-  V(kPointerFieldsStrongEndOffset, 0)                           \
-  /* Weak pointer fields. */                                    \
-  V(kNextCodeLinkOffset, kTaggedSize)                           \
-  V(kPointerFieldsWeakEndOffset, 0)                             \
-  /* Strong Code pointer fields. */                             \
-  V(kCodeOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0) \
-  V(kCodePointerFieldsStrongEndOffset, 0)                       \
-  /* Raw data fields. */                                        \
-  V(kCodeCageBaseUpper32BitsOffset,                             \
-    V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)              \
-  V(kCodeEntryPointOffset,                                      \
-    V8_EXTERNAL_CODE_SPACE_BOOL ? kExternalPointerSize : 0)     \
-  V(kKindSpecificFlagsOffset, kInt32Size)                       \
-  V(kUnalignedSize, OBJECT_POINTER_PADDING(kUnalignedSize))     \
-  /* Total size. */                                             \
+#define CODE_DATA_FIELDS(V)                                         \
+  /* Strong pointer fields. */                                      \
+  V(kPointerFieldsStrongEndOffset, 0)                               \
+  /* Weak pointer fields. */                                        \
+  V(kNextCodeLinkOffset, kTaggedSize)                               \
+  V(kPointerFieldsWeakEndOffset, 0)                                 \
+  /* Strong Code pointer fields. */                                 \
+  V(kCodeOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)     \
+  V(kCodePointerFieldsStrongEndOffset, 0)                           \
+  /* Raw data fields. */                                            \
+  V(kCodeCageBaseUpper32BitsOffset,                                 \
+    V8_EXTERNAL_CODE_SPACE_BOOL ? kTaggedSize : 0)                  \
+  V(kCodeEntryPointOffset,                                          \
+    V8_EXTERNAL_CODE_SPACE_BOOL ? kExternalPointerSize : 0)         \
+  V(kFlagsOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kUInt16Size : 0)    \
+  V(kBuiltinIdOffset, V8_EXTERNAL_CODE_SPACE_BOOL ? kInt16Size : 0) \
+  V(kKindSpecificFlagsOffset, kInt32Size)                           \
+  V(kUnalignedSize, OBJECT_POINTER_PADDING(kUnalignedSize))         \
+  /* Total size. */                                                 \
   V(kSize, 0)
 
   DEFINE_FIELD_OFFSET_CONSTANTS(HeapObject::kHeaderSize, CODE_DATA_FIELDS)
@@ -134,11 +183,27 @@ class CodeDataContainer : public HeapObject {
 
   class BodyDescriptor;
 
+  // Flags layout.
+#define FLAGS_BIT_FIELDS(V, _) \
+  V(KindField, CodeKind, 4, _) \
+  /* The other 12 bits are still free. */
+
+  DEFINE_BIT_FIELDS(FLAGS_BIT_FIELDS)
+#undef FLAGS_BIT_FIELDS
+  STATIC_ASSERT(FLAGS_BIT_FIELDS_Ranges::kBitsCount == 4);
+  STATIC_ASSERT(!V8_EXTERNAL_CODE_SPACE_BOOL ||
+                (FLAGS_BIT_FIELDS_Ranges::kBitsCount <=
+                 FIELD_SIZE(CodeDataContainer::kFlagsOffset) * kBitsPerByte));
+
  private:
   DECL_ACCESSORS(raw_code, Object)
   DECL_RELAXED_GETTER(raw_code, Object)
   inline void set_code_entry_point(Isolate* isolate, Address value);
 
+  // When V8_EXTERNAL_CODE_SPACE is enabled the flags field contains cached
+  // values of some flags of the from the respective Code object.
+  DECL_INT_ACCESSORS(flags)
+
   friend Factory;
   friend FactoryBase<Factory>;
   friend FactoryBase<LocalFactory>;
@@ -698,12 +763,14 @@ class Code::OptimizedCodeIterator {
 // Helper functions for converting Code objects to CodeDataContainer and back
 // when V8_EXTERNAL_CODE_SPACE is enabled.
 inline CodeT ToCodeT(Code code);
+inline Handle<CodeT> ToCodeT(Handle<Code> code, Isolate* isolate);
 inline Code FromCodeT(CodeT code);
 inline Code FromCodeT(CodeT code, RelaxedLoadTag);
 inline Code FromCodeT(CodeT code, AcquireLoadTag);
 inline Code FromCodeT(CodeT code, PtrComprCageBase);
 inline Code FromCodeT(CodeT code, PtrComprCageBase, RelaxedLoadTag);
 inline Code FromCodeT(CodeT code, PtrComprCageBase, AcquireLoadTag);
+inline Handle<CodeT> FromCodeT(Handle<Code> code, Isolate* isolate);
 inline CodeDataContainer CodeDataContainerFromCodeT(CodeT code);
 
 class AbstractCode : public HeapObject {
